@@ -225,7 +225,7 @@ that tries to deal correctly with lexical variables."
      do (setq form (try-pos name (1+ i) form)))))
 
 (declaim (inline read-form))
-(defun read-form ()
+(defun read-form (&key (recursive-p t))
   "Reads and returns one or more Lisp forms from *STREAM* if the
 character we're looking at is a valid inner delimiter. Otherwise
 returns NIL."
@@ -242,10 +242,61 @@ returns NIL."
                  (let ((*readtable* (copy-readtable*)))
                    ;; temporarily change the readtable
                    (set-syntax-from-char end-delimiter #\))
-                   (read-delimited-list end-delimiter *stream* t))))))))
+                   (read-delimited-list end-delimiter *stream* recursive-p))))))))
 
-(defun interpol-reader (*stream* char arg)
-  "The actual reader function for the 'sub-character' #\?."
+(defun read-format-directive ()
+  "Reads and returns a format directive (as a string) along with one
+  or more lisp forms (as per read-form)."
+  (let ((format-directive (make-collector)))
+    (labels ((read-quoted-char ()
+               (if (char= #\' (peek-char*))
+                   (progn
+                    (vector-push-extend (read-char*) format-directive)
+                    (vector-push-extend (read-char*) format-directive)
+                    t)
+                   nil))
+             (read-integer ()
+               (if (member (peek-char*) '(#\- #\0 #\1 #\2 #\3 #\4 #\5 #\6 #\7 #\8 #\9))
+                   (progn
+                     (vector-push-extend (read-char*) format-directive)
+                     (loop while (member (peek-char*) '(#\- #\0 #\1 #\2 #\3 #\4 #\5 #\6 #\7 #\8 #\9))
+                           do (vector-push-extend (read-char*) format-directive))
+                     t)
+                   nil))
+             (read-modifier ()
+               (loop repeat 2
+                     with found = nil
+                     when (member (peek-char*) '(#\@ #\:))
+                       do (vector-push-extend (read-char*) format-directive)
+                       and do (setf found t)
+                     finally (return found)))
+             (read-comma ()
+               (if (char= #\, (peek-char*))
+                   (progn
+                     (vector-push-extend (read-char*) format-directive)
+                     t)
+                   nil))
+             (read-v ()
+               (if (char-equal #\v (peek-char*))
+                   (progn
+                     (vector-push-extend (read-char*) format-directive)
+                     t)
+                   nil)))
+      (loop
+        while (or (read-quoted-char)
+                  (read-integer)
+                  (read-v)
+                  (read-comma))
+        finally (read-modifier)
+        finally (vector-push-extend (read-char*) format-directive))
+      format-directive)))
+
+(defun interpol-reader (*stream* char arg &key (recursive-p t))
+  "The actual reader function for the 'sub-character' #\?.
+
+This function can be used directly outside of a read table by passing `recursive-p` as NIL.
+
+"
   (declare (ignore arg char))
   (let ((*start-char* (read-char*))
         ;; REGEX-MODE is true if we're in regular expression mode; it
@@ -280,11 +331,11 @@ returns NIL."
           *saw-backslash*
           *readtable-copy*)
       (prog1
-        (inner-reader regex-mode extended-mode nil nil)
+        (inner-reader regex-mode extended-mode nil nil :recursive-p recursive-p)
         ;; consume the closing outer delimiter
         (read-char*)))))
 
-(defun inner-reader (regex-mode extended-mode quote-mode case-mode)
+(defun inner-reader (regex-mode extended-mode quote-mode case-mode &key (recursive-p t))
   "Helper function for INTERPOL-READER which does all the work. May
 call itself recursively."
   ;; REGEX-MODE and EXTENDED-MODE as described above; QUOTE-MODE is
@@ -540,9 +591,19 @@ call itself recursively."
                                     ;; otherwise this is a
                                     ;; backslash-escaped character
                                     (unescape-char regex-mode))))
+                              ((#\~)
+                               ;; #\~ - might be an inline format directive
+                               (if *interpolate-format-directives*
+                                   `(format ,string-stream
+                                            ,(concatenate 'string "~" (read-format-directive))
+                                            ,@(let ((form (read-form :recursive-p recursive-p)))
+                                                (if form
+                                                    (list form)
+                                                    '())))
+                                   #\~))
                               ((#\$)
                                 ;; #\$ - might be an interpolation
-                                (let ((form (read-form)))
+                                (let ((form (read-form :recursive-p recursive-p)))
                                   (cond ((null form)
                                           ;; no, just dollar sign
                                           #\$)
@@ -570,7 +631,7 @@ call itself recursively."
                                           `(princ ,form ,string-stream)))))
                               ((#\@)
                                 ;; #\Q - might be an interpolation
-                                (let ((form (read-form))
+                                (let ((form (read-form :recursive-p recursive-p))
                                       (element (gensym))
                                       (first (gensym)))
                                   (cond ((null form)
@@ -687,12 +748,13 @@ call itself recursively."
                 else
                 collect interpolation)))))
 
-(defun %enable-interpol-syntax ()
+(defun %enable-interpol-syntax (&key (modify-*readtable* nil))
   "Internal function used to enable reader syntax and store current
 readtable on stack."
-  (push *readtable*
-        *previous-readtables*)
-  (setq *readtable* (copy-readtable))
+  (unless modify-*readtable*
+    (push *readtable*
+          *previous-readtables*)
+    (setq *readtable* (copy-readtable)))
   (set-dispatch-macro-character #\# #\? #'interpol-reader)
   (values))
 
@@ -703,14 +765,14 @@ readtable on stack."
     (setq *readtable* (copy-readtable nil)))
   (values))
 
-(defmacro enable-interpol-syntax ()
+(defmacro enable-interpol-syntax (&rest %enable-interpol-syntax-args)
   "Enable CL-INTERPOL reader syntax."
   `(eval-when (:compile-toplevel :load-toplevel :execute)
-    (%enable-interpol-syntax)))
+    (%enable-interpol-syntax ,@%enable-interpol-syntax-args)))
 
 (defmacro disable-interpol-syntax ()
   "Restore readtable which was active before last call to
 ENABLE-INTERPOL-SYNTAX. If there was no such call, the standard
 readtable is used."
   `(eval-when (:compile-toplevel :load-toplevel :execute)
-    (%disable-interpol-syntax)))
\ No newline at end of file
+    (%disable-interpol-syntax)))
